package kernel.analyzer;

import java.util.ArrayList;
import java.util.List;

/**
 * Description: Object properties for setting commands.
 * 
 * @author david.kosek
 * @date 13. 11. 2014 18:22:28
 * @version $Rev$
 */
public class InputParseObject {

	/**
	 * Declaration flag for input
	 */
	private boolean stdInFlag = false;

	/**
	 * Declaration variable for name of file
	 */
	private String stdInNameFile = null;

	/**
	 * Declaration flag for output
	 */
	private boolean stdOutFlag = false;

	/**
	 * Declaration variable for name of file
	 */
	private String stdOutNameFile = null;

	/**
	 * Declaration flag for append
	 */
	private boolean appendOutFlag = false;

	/**
	 * Declaration variable for name of command
	 */
	private String command = null;

	/**
	 * Declaration list of parametrs and arguments
	 */
	private List<String> parametsAndArgumentsList = new ArrayList<String>();

	/**
	 * Constructor for new command
	 * 
	 * @param aStdInFlag true - if is setting standard input else false
	 * @param aStdInNameFile name of file for input
	 * @param aStdOutFlag true - if is setting standard output else false
	 * @param aStdOutNameFile name of file for output
	 * @param aAppendOutFlag true - if is setting append to exists file else false
	 * @param aCommand name of command
	 * @param aParametsAndArgumentsList list of parameters/arguments for command
	 */
	public InputParseObject(
			boolean aStdInFlag,
			String aStdInNameFile,
			boolean aStdOutFlag,
			String aStdOutNameFile,
			boolean aAppendOutFlag,
			String aCommand,
			List<String> aParametsAndArgumentsList) {
		this.stdInFlag = aStdInFlag;
		this.stdInNameFile = aStdInNameFile;
		this.stdOutFlag = aStdOutFlag;
		this.stdOutNameFile = aStdOutNameFile;
		this.appendOutFlag = aAppendOutFlag;
		this.command = aCommand;
		this.parametsAndArgumentsList = aParametsAndArgumentsList;
	}

	/**
	 * Flag for standard input
	 * 
	 * @return flag if is setting input
	 */
	public boolean isStdInFlag() {
		return stdInFlag;
	}

	/**
	 * Name of file for standard input
	 * 
	 * @return name of file
	 */
	public String getStdInNameFile() {
		return stdInNameFile;
	}

	/**
	 * Flag for standard output
	 * 
	 * @return flag if is setting output
	 */
	public boolean isStdOutFlag() {
		return stdOutFlag;
	}

	/**
	 * Name of file for standard output
	 * 
	 * @return name of file
	 */
	public String getStdOutNameFile() {
		return stdOutNameFile;
	}

	/**
	 * Flag for append text to file
	 * 
	 * @return flag if is setting append
	 */
	public boolean isAppendOutFlag() {
		return appendOutFlag;
	}

	/**
	 * Name of command
	 * 
	 * @return command name
	 */
	public String getCommand() {
		return command;
	}

	/**
	 * List of arguments/parametrs for current command
	 * 
	 * @return list of arguments/parametrs
	 */
	public List<String> getParametsAndArgumentsList() {
		return parametsAndArgumentsList;
	}

}
