package kernel.analyzer;

import java.util.Arrays;
import java.util.regex.Pattern;

/**
 * Description: Check current input from user according grammar.
 * 
 * @author david.kosek
 * @date 9.10.2014 23:14:42
 * @version $Rev$
 */
public class SyntacticAnalysis implements ISyntactitAnalysis {

	/**
	 * Declaration flag for output
	 */
	private boolean usedLeftArrow;

	/**
	 * Declaration flag for input
	 */
	private boolean usedRightArrow;

	/**
	 * Declaration array for simple text
	 */
	private String[] splitArrayIntoSimpleText;

	@Override
	public boolean checkInput(String aInput) {

		// because trim doesn't work
		aInput = aInput.replaceAll("^\\s+|\\s+$", "");

		if (aInput.length() == 0) {
			return true;
		}

		String[] array = null;
		array = aInput.split(" ");

		// check/remove if is correct array
		array = removeEmptyElements(array);

		// split array according conditions in taks
		array = split(array);

		usedLeftArrow = false;
		usedRightArrow = false;

		// check if first string is word
		if (!checkValueIsWord(array[0])) {
			return false;
		}

		for (int i = 1; i < array.length; i++) {

			// check argument start with "- and char"
			if (SchemaMetaDataCommand.BASIC_SYMBOL.DASH.equals(array[i].substring(0, 1))) {
				if (!(array[i].length() == 2)) {
					System.err.println("Argument can't be only '-' (missing char). Use help about this.");
					return false;
				}
			}

			// check std. input and exist file as parametr
			if ((SchemaMetaDataCommand.BASIC_SYMBOL.LEFT_ARROW.equals(array[i])) && !usedRightArrow) {
				if (!usedLeftArrow) {
					if (!(array.length - 1 >= i + 1)) {
						System.err.println("Standart input can't be only '<' (missing file). Use help about this.");
						return false;
					}
					// maybe delete it, we can discuss about this
					if (!(array[i + 1].length() != 1)) {
						System.err.println("File length can't be so short. Use help about this.");
						return false;
					}
					usedLeftArrow = true;
				}
				else {
					System.err.println("You can't use twice std. input in one command. Use help about this.");
					return false;
				}

			}

			// check output and exist file as parametr
			if ((SchemaMetaDataCommand.BASIC_SYMBOL.RIGHT_ARROW.equals(array[i]) || SchemaMetaDataCommand.BASIC_SYMBOL.DOUBLE_RIGHT_ARROW.equals(array[i]))) {
				if (!usedRightArrow) {
					if (!(array.length - 1 >= i + 1)) {
						System.err.println("Standart output can't be only '>' (missing file). Use help about this.");
						return false;
					}
					// maybe delete it, we can discuss about this
					if (!(array[i + 1].length() != 1)) {
						System.err.println("File length can't be so short. Use help about this.");
						return false;
					}

					usedRightArrow = true;
				}
				else {
					System.err.println("You can't use twice std. output in one command. Use help about this.");
					return false;
				}
			}

			// check if ampersand is last command, DELETE
			if (SchemaMetaDataCommand.BASIC_SYMBOL.AMPERSAND.equals(array[i])) {
				if (!(array.length == i)) {
					return false;
				}
			}

			// check pipe and continue in string
			if (SchemaMetaDataCommand.BASIC_SYMBOL.PIPE.equals(array[i])) {
				if (!(array.length - 1 >= i + 1)) {
					System.err.println("You can't end with only pipe. Use help about this.");
					return false;
				}

				if (!checkValueIsWord(array[i + 1])) {
					System.err.println("After pipe must be command. Use help about this.");
					return false;
				}

				usedLeftArrow = false;
				usedRightArrow = false;
			}

			// check if has current text length equal 1
			if (array[i].length() == 1 && !checkValueIsWord(array[i]) && !SchemaMetaDataCommand.BASIC_SYMBOL.AMPERSAND.equals(array[i])
					&& !SchemaMetaDataCommand.BASIC_SYMBOL.PIPE.equals(array[i])
					&& !SchemaMetaDataCommand.BASIC_SYMBOL.RIGHT_ARROW.equals(array[i])
					&& !SchemaMetaDataCommand.BASIC_SYMBOL.DOUBLE_RIGHT_ARROW.equals(array[i])
					&& !SchemaMetaDataCommand.BASIC_SYMBOL.LEFT_ARROW.equals(array[i])
					&& !SchemaMetaDataCommand.BASIC_SYMBOL.SLASH.equals(array[i]) && !SchemaMetaDataCommand.BASIC_SYMBOL.DOT.equals(array[i])
					&& !SchemaMetaDataCommand.BASIC_SYMBOL.BACKSLASH.equals(array[i]) && !checkValueIsNumber(array[i])) {
				System.err.println("If has part of command length equal 1 mustn't be special character. Use help about this.");
				return false;
			}

		}
		this.splitArrayIntoSimpleText = array;
		return true;
	}

	/**
	 * Remove empty elements from array
	 * 
	 * @param aArray current input in array
	 * @return array without empty elements
	 */
	private String[] removeEmptyElements(String[] aArray) {
		int r, w;
		final int n = r = w = aArray.length;
		while (r > 0) {
			final String s = aArray[--r];
			if (!s.isEmpty()) {
				aArray[--w] = s;
			}
		}
		return Arrays.copyOfRange(aArray, w, n);
	}

	/**
	 * Split array according whitespace and necessary conditions
	 * 
	 * @param aArray current input in array
	 * @return correct split array
	 */
	private String[] split(String[] aArray) {

		String[] newArray = new String[aArray.length];
		String specialChar = null;
		String newParam = null;

		int oldPosition = 0;
		int newPosition = 0;
		while (true) {

			try {

				if (aArray[oldPosition].length() == 1) {
					newArray[newPosition] = aArray[oldPosition];
				}
				else if (SchemaMetaDataCommand.BASIC_SYMBOL.QUOTATION_MARKS.equals(aArray[oldPosition].substring(0, 1))) {
					String textInMarks = aArray[oldPosition];
					Integer position = 0;
					Boolean moreTextInMark = true;
					if (SchemaMetaDataCommand.BASIC_SYMBOL.QUOTATION_MARKS.equals(aArray[oldPosition].substring(
							aArray[oldPosition].length() - 1,
							aArray[oldPosition].length()))) {
						moreTextInMark = false;
					}

					while (moreTextInMark) {
						position++;
						oldPosition++;
						if (oldPosition > aArray.length - 1) {
							System.err.println("The text in quotation marks must be correctly closed. Support team fixed this. Use help next time.");
							textInMarks += "\"";
							moreTextInMark = false;
							position--;
						}
						else if (aArray[oldPosition].contains("\"")) {
							textInMarks += " " + aArray[oldPosition];
							moreTextInMark = false;
						}
						else {
							textInMarks += " " + aArray[oldPosition];
						}
					}
					// remove quotation marks from text
					textInMarks = textInMarks.substring(1, textInMarks.length() - 1);
					newArray = resizeArray(newArray, position);
					newArray[newPosition] = textInMarks;
				}
				else if ((SchemaMetaDataCommand.BASIC_SYMBOL.LEFT_ARROW.equals(aArray[oldPosition].substring(0, 1))
						|| SchemaMetaDataCommand.BASIC_SYMBOL.PIPE.equals(aArray[oldPosition].substring(0, 1)) || (SchemaMetaDataCommand.BASIC_SYMBOL.RIGHT_ARROW.equals(aArray[oldPosition].substring(
						0,
						1)) && !SchemaMetaDataCommand.BASIC_SYMBOL.DOUBLE_RIGHT_ARROW.equals(aArray[oldPosition].substring(0, 2))))
						&& (aArray[oldPosition].length() > 1)) {
					specialChar = aArray[oldPosition].substring(0, 1);
					newParam = aArray[oldPosition].substring(1, aArray[oldPosition].length());
					newArray = copyAndEnlargeArray(newArray);
					newArray[newPosition] = specialChar;
					newArray[newPosition + 1] = newParam;
					newPosition++;
				}
				else if (SchemaMetaDataCommand.BASIC_SYMBOL.DOUBLE_RIGHT_ARROW.equals(aArray[oldPosition].substring(0, 2))
						&& (aArray[oldPosition].length() > 2)) {
					specialChar = aArray[oldPosition].substring(0, 2);
					newParam = aArray[oldPosition].substring(2, aArray[oldPosition].length());
					newArray = copyAndEnlargeArray(newArray);
					newArray[newPosition] = specialChar;
					newArray[newPosition + 1] = newParam;
					newPosition++;
				}
				else {
					newArray[newPosition] = aArray[oldPosition];
				}

				newPosition++;
				oldPosition++;
			}
			catch (IndexOutOfBoundsException e) {
				break;
			}
		}

		return newArray;
	}

	/**
	 * Reduction array
	 * 
	 * @param aNewArray array for copy to new array
	 * @param aPosition number by how much diminish array
	 * @return reduction new array
	 */
	private String[] resizeArray(String[] aNewArray, int aPosition) {
		String[] newArray = new String[aNewArray.length - aPosition];

		for (int i = 0; i < newArray.length; i++) {
			newArray[i] = aNewArray[i];
		}

		return newArray;
	}

	/**
	 * Enlargement array
	 * 
	 * @param aNewArray array for copy to new array
	 * @return enlargement new array
	 */
	private String[] copyAndEnlargeArray(String[] aNewArray) {
		String[] newArray = new String[aNewArray.length + 1];

		for (int i = 0; i < aNewArray.length; ++i) {
			newArray[i] = aNewArray[i];
		}

		return newArray;
	}

	@Override
	public String[] getSplitArrayIntoSimpleText() {
		return splitArrayIntoSimpleText;
	}

	@Override
	public boolean checkValueIsWord(String aWord) {
		return Pattern.matches("[a-zA-Z]+", aWord);
	}

	@Override
	public boolean checkValueIsNumber(String aNumber) {
		return Pattern.matches("[0-9]+", aNumber);
	}

}
