package kernel.file;

import java.io.File;
import java.io.PipedInputStream;
import java.util.List;

/**
 * Description: Interface for file manager for work with file.
 * 
 * @author david.kosek
 * @date 2. 12. 2014 15:17:34
 * @version $Rev$
 */
public interface IFileManager {

	/**
	 * @param absolutePath complete path to file
	 * @return instance of file or null
	 */
	File getFile(final String aAbsolutePathToFile);

	/**
	 * Loading actual path
	 * 
	 * @return actual path
	 */
	String getCurrentPath();

	/**
	 * Create dir/folder in actual place
	 * 
	 * @param nameFolder name for new folder
	 * @return true if was created dir/folder and didn't exist, else false
	 */
	Boolean createFolder(final String aNameFolder);

	/**
	 * Loading all files from request path, if is empty will be use actual path
	 * 
	 * @param aPath path where is dir/folder
	 * @return list of all files
	 */
	List<File> loadAllFileFromDir(final String aPath);

	/**
	 * Loading all folder from request path, if is empty will be use actual path
	 * 
	 * @param aPath path where is dir/folder
	 * @return list of all folder/dir
	 */
	List<File> loadAllFolderFromDir(final String aPath);

	/**
	 * Loading all from request path, if is empty will be use actual path
	 * 
	 * @param aPath path where is dir/folder
	 * @return list of all folder/dir and file
	 */
	List<File> loadAllFromDir(final String aPath);

	/**
	 * Loading all from request path, if is empty will be use actual path including hidden things
	 * 
	 * @param aPath path where is dir/folder
	 * @return list of all folder/dir and file including hidden things
	 */
	List<File> loadAllAndHiddenFromDir(final String aPath);

	/**
	 * Open file for write and set necessary properties
	 * 
	 * @param aNameFile name of file
	 * @param aAppend true if I want append text to end file
	 * @param aPipeIn input piped stream
	 * @return true if it is all correctly
	 */
	Boolean openFileForWrite(String aNameFile, Boolean aAppend, PipedInputStream aPipeIn);

	/**
	 * Write text to current file
	 * 
	 * @param aText request text
	 */
	void writeToFile(String aText);

	/**
	 * Close file which is opening actual
	 */
	void closeWriteToFile();

	/**
	 * Read line from file
	 * 
	 * @return new line
	 */
	String readFromFile();

	/**
	 * Close file which is opening actual
	 */
	void closeReadToFile();

	/**
	 * Open file for read and set necessary properties
	 * 
	 * @param aNameFile name of file
	 * @return true if it is all correctly
	 */
	Boolean openFileForRead(String aNameFile);

	/**
	 * Delete file if he exists
	 * 
	 * @param aName name of file
	 * @param aRecursively true/false
	 * @return true if file was deleted else false
	 */
	boolean deleteFile(String aName, boolean aRecursively);

}
