package kernel.file;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.UnsupportedEncodingException;

import kernel.service.CustomException;

/**
 * Description: Input utility for read text from file.
 * 
 * @author david.kosek
 * @date 2. 12. 2014 15:47:43
 * @version $Rev$
 */
public class InputUtility implements IInputUtility {

	/**
	 * Declarate input stream
	 */
	private PipedInputStream inPiped;

	/**
	 * Declarate buffer for read
	 */
	private BufferedReader input;

	/**
	 * Declarate charset
	 */
	public final String CHARSET = "UTF-8";

	/**
	 * Declarate file reader
	 */
	private FileInputStream fileReader;

	/**
	 * Constructor
	 * 
	 * @param aFile name of file
	 */
	public InputUtility(String aFile) {
		try {
			File file = new File(aFile);
			fileReader = new FileInputStream(file);
			input = new BufferedReader(new InputStreamReader(fileReader, CHARSET));
		}
		catch (FileNotFoundException | UnsupportedEncodingException e) {
			CustomException.printException(e, "Cannot read from file.");
		}
	}

	/**
	 * Constructor
	 * 
	 * @param aOut output stream
	 * @param aFile name of file
	 */
	public InputUtility(PipedOutputStream aOut, String aFile) {
		try {
			inPiped = new PipedInputStream();
			inPiped.connect(aOut);

			File file = new File(aFile);
			fileReader = new FileInputStream(file);
			input = new BufferedReader(new InputStreamReader(fileReader, CHARSET));
		}
		catch (Exception e) {
			CustomException.printException(e, "Cannot read from file.");
		}
	}

	@Override
	public String read() {
		try {
			return input.readLine();
		}
		catch (IOException e) {
			CustomException.printException(e, "End of file.");
			return null;
		}
	}

	@Override
	public void close() {
		try {
			input.close();
		}
		catch (IOException e) {
			CustomException.printException(e, "File wasn't closed right.");
		}
	}

}
