package kernel.file;

import java.io.FileWriter;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintWriter;

import kernel.service.CustomException;

public class OutputUtility implements IOutputUtility {

	/**
	 * Declarate output stream
	 */
	private PipedOutputStream outputPipe;
	/**
	 * Declarate charset
	 */
	public final String CHARSET = "UTF-8";
	/**
	 * Declarate print writer
	 */
	private PrintWriter out;

	/**
	 * Declarate flag for new file or append text
	 */
	private Boolean newOrAppend = false;

	/**
	 * Contructor
	 * 
	 * @param aFile name of file
	 * @param isNew true if file is exists and I want append text
	 */
	public OutputUtility(String aFile, Boolean isNew) {
		try {
			this.out = new PrintWriter(new FileWriter(aFile, isNew));
			this.newOrAppend = isNew;
		}
		catch (Exception e) {
			CustomException.printException(e, "Cannot write from file.");
		}
	}

	/**
	 * Contructor
	 * 
	 * @param aIn piped input stream
	 * @param aFile name of file
	 * @param isNew true if file is exists and I want append text
	 */
	public OutputUtility(PipedInputStream aIn, String aFile, Boolean isNew) {
		try {
			this.outputPipe = new PipedOutputStream();
			aIn.connect(outputPipe);

			this.out = new PrintWriter(new FileWriter(aFile, isNew));
			this.newOrAppend = isNew;
		}
		catch (Exception e) {
			CustomException.printException(e, "Cannot write from file.");
		}
	}

	@Override
	public void write(String text) {

		try {
			if (!newOrAppend) {
				out.write(text);
				out.flush();
			}
			else {
				out.append(text);
				out.flush();
			}
		}
		catch (Exception e) {
			CustomException.printException(e, "Cannot write from file.");
		}
	}

	@Override
	public void close() {
		out.flush();
		out.close();
	}

}
