package kernel.process;

import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import kernel.process.sbin.Init;
import kernel.process.sbin.Shell;
import kernel.terminal.Terminal;

/**
 * Class for manager of processes
 *
 */
public class ProcessManager {
	/**
	 * Instance of ProcessManager
	 */
	private static ProcessManager processManager;
	/**
	 * Processes id
	 */
	private static int processID;
	/**
	 * Map of processes
	 */
	private Map<Integer, IProcess> processes;
	/**
	 * Constructor for create instance	
	 */
	private ProcessManager(){
		processID = 1;
		processes = new HashMap<Integer, IProcess>();
	};
	
	/**
	 * Get instance of ProcessManager
	 * 
	 * @return Instance ProcessManager
	 */
	public static ProcessManager getProcessManager() {
		if (processManager == null) {
			processManager = new ProcessManager();
		}
		return processManager;
	}
	
	/**
	 * Create a new process and added its to MashMap of all processes
	 * 
	 * @return Integer PID
	 */
	public int createProcess(IProcess process, IProcess parent){
		process.setPID(processID);
		processID++;
		if (parent != null){
			process.setPPID(parent.getPID());
		}
		else {
			process.setPPID(-1);
		}
		processes.put(process.getPID(), process);
		process.start();		
		return process.getPID();
	}
	
	/**
	 * Method get list of processes
	 * 
	 * @return list of processes
	 */
	public List<String> getListProcess() {
		List<String> list = new LinkedList<String>();
		
		for (Map.Entry<Integer, IProcess> entry : processes.entrySet()) {
		    list.add(
		    		entry.getValue().getPID() + 
		    		";" + 
		    		entry.getValue().getPPID() + 
		    		";" + 
		    		entry.getValue().getClass().getSimpleName());
		}	
		return list;
	}
	
	/**
	 * Correctly terminated process.
	 * 
	 * @param PID Process ID
	 * @return Integer status
	 */
	public void closeProcess(int PID){
		IProcess deleteProcess = processes.get(PID);
		if (deleteProcess != null) {
			for (Map.Entry<Integer, IProcess> entry : processes.entrySet()) {
			    if (entry.getValue().getPPID() == deleteProcess.getPID()) {
			    	entry.getValue().setPPID(deleteProcess.getPPID());
			    }
			}		
			processes.remove(deleteProcess.getPID());	
		}
	}
	
	/**
	 * Method for get instance of Shell
	 * 
	 * @return instance of Shell
	 */
	public IProcess getShell() {
		for (Map.Entry<Integer, IProcess> entry : processes.entrySet()) {
		    if (entry.getValue().getClass().getSimpleName().equals("Shell")) {
		    	return entry.getValue();
		    }
		}
		return null;
	}
	
	/**
	 * Forced terminated process.
	 * 
	 * @param PID Process ID
	 */
	@SuppressWarnings("deprecation")
	public boolean killProcess(int PID){
		if (processes.get(PID) instanceof Init ||  processes.get(PID) instanceof Shell) {
			return false;
		}
		if (processes.get(PID) == null) {
			return false;
		}
		((Thread) processes.get(PID)).stop();
		return true;
	}
	/**
	 * Method for close of OS model
	 */
	public void shutdown() {	
		ArrayList<IProcess> processesSort = new ArrayList<IProcess>();
		
		for (Map.Entry<Integer, IProcess> entry : processes.entrySet()) {
			processesSort.add(entry.getValue());
		}	
		for (int i = processesSort.size()-1; i >= 0; i--) {
			if (processesSort.get(i) instanceof Shell) {
				((Shell)processesSort.get(i)).shutdown();
				((Thread) processesSort.get(i)).interrupt();
			}
			Terminal.getTerminal().shudown();	
		}
	}
}
