package kernel.process.bin;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import kernel.file.FileManager;
import kernel.process.Process;

/**
 * The program lists all files and folders in the specified directory.
 */
public class Ls extends Process {


	@Override
	public void run() {
		String text = "";
		List<File> files = null;
		String path = getRelation().getPath();
		FileManager fm = new FileManager();
		if (getParametsAndArguments().size() == 0) {
			files = fm.loadAllFromDir(path);
		}
		else {
			for (String p : getParametsAndArguments()) {
				
				switch (p) {
				case "-a":
				case "--all":
					files = fm.loadAllAndHiddenFromDir(path);
					break;
				case "-l":
					files = fm.loadAllFromDir(path);
					break;
				case "-f":
				case "--files":
					files = fm.loadAllFileFromDir(path);
					break;
				case "-d":
				case "--directories":
					files = fm.loadAllFolderFromDir(path);
					break;
				default:
					if(p.contains("-") || p.contains("--")){
						text += "Neznm parametr\n";
					}else {
						path = new File(p).isAbsolute() ? p : (path + p); //cestu vzdy prevedu na absolutni
						try {
							files = fm.loadAllFromDir(path);
						}
						catch(NullPointerException e){
							System.err.println("patn cesta | " + e.getClass() + ": \n");
							e.printStackTrace();
						}
					}
					break;
				}
			}
		}
		
		if(files == null){
			text += "Cannot read this directory.\n";
		}else if(files.isEmpty()) {
			text += "No files or directories in this path.\n";
		}else if(getParametsAndArguments().contains("-l")){
			text += "Type\tRight\tName\tSize\tLast modified\n";
			text += "--------------------------------------------------------------------------------------------------------------------\n";
			for(File f : files){
				text += f.isDirectory()?"D":"";
				text += f.isFile()?"F":"";
				text += f.isHidden()?"H":"";

				text += "\t";
				text += f.canRead() ? "x" : "-";
				text += f.canWrite() ? "x" : "-";
				text += f.canExecute() ? "x" : "-";
				text += "\t";
				text += f.getName();
				text += "\t";
				text += f.isFile() ? f.length() : "-----";
				text += "\t";
				text += new SimpleDateFormat("dd-MM-yyyy HH-mm-ss").format(
					    	new Date(f.lastModified()) 
						);
				text += "\n";
			}

		}else {
			for(File f : files){
				 text += f.getName() + "\n";
			}
		}
		getOutput().write(text);
	}
}
