package kernel.process.bin;

import java.io.File;
import java.io.IOException;
import java.nio.file.CopyOption;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;

import kernel.process.Process;
import kernel.service.CustomException;

/**
 * The program move or rename file (files) or directory (directories)
 */
public class Mv extends Process {

	@Override
	public void run() {
		String text = "";
		String path = getRelation().getPath();
		String p;
		if (getParametsAndArguments().size() == 0) {
			text += "Nothing to move.\n";
		}
		else if (getParametsAndArguments().size() < 2){
			text += "Missing destination.\n";
		}
		else {
			//get source destination as string
			p = getParametsAndArguments().get(0);
			//convert relative string path to absolute path a get Path type
			Path FROM = Paths.get(new File(p).isAbsolute() ? p : (path + p));
			p = getParametsAndArguments().get(1);
		    Path TO = Paths.get(new File(p).isAbsolute() ? p : (path + p));
		    
		    CopyOption[] options = new CopyOption[]{
		      StandardCopyOption.REPLACE_EXISTING
		    }; 
		    try {
				Files.move(FROM, TO, options);
				if (FROM.toFile().isDirectory()) {
				    for (File c : FROM.toFile().listFiles()){
				    	Files.move(c.toPath(), new File(TO.toFile(), c.getName()).toPath(), options);
				    }
			    }
				text += "MOVE successful\n";
			} catch (IOException e) {
				CustomException.printException(e, "Cannot copy this file!");
			}
		}

		getOutput().write(text);
	}
}
