package kernel.process.sbin;

import java.io.File;
import java.util.List;

import kernel.analyzer.InputParseObject;
import kernel.analyzer.SemanticAnalysis;
import kernel.analyzer.SyntacticAnalysis;
import kernel.analyzer.ISemanticAnalysis;
import kernel.analyzer.ISyntactitAnalysis;
import kernel.file.InputFileForReadStream;
import kernel.file.OutputFileForWriteStream;
import kernel.process.IProcess;
import kernel.process.Process;
import kernel.process.ProcessManager;
import kernel.process.ProgramInitializer;
import kernel.relation.Relation;
import kernel.relation.RelationManager;
import kernel.service.Input;
import kernel.service.InputOutputPipe;
import kernel.service.Output;
import kernel.terminal.Terminal;

/**
 * Class for start of processes
 *
 */
public class Shell extends Process {
	/**
	 * Count of shell
	 */
	private static int countShell = 0;
	/**
	 * Syntactit analyzer
	 */
	private ISyntactitAnalysis syntactitAnalyzer = new SyntacticAnalysis();
	/**
	 * Sematic analyzer
	 */
	private ISemanticAnalysis semanticAnalyzer = new SemanticAnalysis();

	/**
	 * Live of thread
	 */
	private boolean live = true;

	@Override
	public void run() {
		countShell++;
		RelationManager.getRelationManager().createRelation(getPID());
		Terminal.getTerminal().setShellID(getPID());

		Relation relation = RelationManager.getRelationManager().getRelation(getPID());
		String path = new File(".").getAbsolutePath();
		path += File.separator;
		relation.setPath(path.substring(0, path.length() - 2));
		
		setOutput(new Output(Terminal.getTerminal().getInput()));
		setInput(new Input(Terminal.getTerminal().getOutput()));
		
		while (live) {	
			String n = relation.getPath() + "> ";
			getOutput().write(n);

			String line = getInput().readLine();
			if (line != null) {
				runCommnad(line.trim());
			}
				
			syntactitAnalyzer = new SyntacticAnalysis();
			semanticAnalyzer = new SemanticAnalysis();
		}
	}

	/**
	 * Method for running process
	 * 
	 * @param allCommand
	 */
	public void runCommnad(String allCommand) {
		if (allCommand.equals("shell")) {
			Shell newShell = new Shell();
			
			Terminal.getTerminal().commandNewShell();
			ProcessManager.getProcessManager().createProcess(newShell, this);
			return;
		}

		if (allCommand.equals("exit")) {
			countShell--;
			if (countShell > 0) {
				ProcessManager.getProcessManager().closeProcess(getPID());
			}
			Terminal.getTerminal().closeShell();
			live = false;
			return;
		}

		if (allCommand.equals("")) {
			return;
		}

		if (syntactitAnalyzer.checkInput(allCommand)) {
			semanticAnalyzer.setValuesAndCheckCommand(syntactitAnalyzer.getSplitArrayIntoSimpleText());

			List<InputParseObject> listCommmand = semanticAnalyzer.getResultInputList();
						
			InputOutputPipe pipe = null;
			InputOutputPipe io = new InputOutputPipe();
			
			IProcess predchozoProces = this;
			for (int i = 0; i < listCommmand.size(); i++) {
				
				String oneCommand = listCommmand.get(i).getCommand();
				if (oneCommand.equals("")) {return;}

				IProcess process = ProgramInitializer.getProcess(oneCommand);

				if (process == null) {
					String userCommand = "Unidentified command\n";
					getOutput().write(userCommand);
					return;
				}
				
				Input inputa;
				if (listCommmand.get(i).isStdInFlag()) {
					inputa = new Input(io.getPis());
					InputFileForReadStream.InputFileForReadStreamStatic(listCommmand.get(i).getStdInNameFile(), io.getPos());

				} else {
					if (i == 0) {
						inputa = getInput();
					} else {
						inputa = new Input(pipe.getPis());
					}
				}	
				process.setInput(inputa);
				
				pipe = new InputOutputPipe();
				
				Output out;
				if (!listCommmand.get(i).isStdOutFlag() && !listCommmand.get(i).isAppendOutFlag()) {
					out = getOutput();
					if (i == listCommmand.size()-1) {
						out = getOutput();
					} else {
						out = new Output(pipe.getPos());
					}
				} else {
					out = new Output(io.getPos());
				}
				process.setOutput(out);
														
				process.setParametsAndArguments(listCommmand.get(i).getParametsAndArgumentsList());
				process.setRelation(RelationManager.getRelationManager().getRelation(getPID()));
				
				ProcessManager.getProcessManager().createProcess(process, predchozoProces);
				predchozoProces = process;
				try { 
					((Thread) process).join(); 
					if (listCommmand.get(i).isStdOutFlag() || listCommmand.get(i).isAppendOutFlag()) {
						OutputFileForWriteStream.OutputFileForWriteStreamStatic(
								listCommmand.get(i).getStdOutNameFile(), 
								listCommmand.get(i).isAppendOutFlag(),
								io.getPis());
					}
				} catch (InterruptedException e) { 
					//ignore - process was close
				}	
				ProcessManager.getProcessManager().closeProcess(process.getPID());
			}
		} else {
			getOutput().write("Wrong command\n");
		}

	}
	/**
	 * Close process
	 */
	public void shutdown() {
		live = false;
	}
}

