package kernel.terminal;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import kernel.relation.Relation;
import kernel.relation.RelationManager;
import kernel.service.InputOutputPipe;

/**
 * Class for create terminal
 */
public class Terminal extends JFrame {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Class for preserved date of shell process
	 */
	class DataForShell {
		JPanel panel;		
		InputReader ir;
		InputOutputPipe input;
		InputOutputPipe output;
		InputStreamReader is;
		JTextArea text;
		int shellID;
		int idStartCommand = 0;
		int idCommand = 1;
		int posCaret;
	}
	
	
	private JTabbedPane mainJTabbedPane = new JTabbedPane();
		
	private int idShell = 1;	
	private static Terminal terminal;
	
	private Map<String, DataForShell> dataForShellMap = new HashMap<String, DataForShell>();
	private DataForShell currentdataForShell = new DataForShell();
	
	/**
	 * Create instance of terminal
	 */
	private Terminal() {	
		setTitle("Simulating operation system in Java ");
		setSize(700, 500);
		setLocationRelativeTo(null);
		
		getContentPane().add(createMainPanel());
		
		mainJTabbedPane.addChangeListener(new ChangeListener() {
			
			@Override
			public void stateChanged(ChangeEvent arg0) {			
				JTabbedPane pane = (JTabbedPane)arg0.getSource();
				currentdataForShell = dataForShellMap.get(pane.getSelectedComponent().getName()); 
				currentdataForShell.idStartCommand = currentdataForShell.text.getText().length();
				currentdataForShell.text.setCaretPosition(currentdataForShell.idStartCommand);
			}
		});		
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		setVisible(true);		
	}
	
	/**
	 * Method for get instance of terminal
	 * 
	 * @return instance of terminal
	 */
	public static Terminal getTerminal() {
		if (terminal == null) {
			terminal = new Terminal();
		}
		return terminal;
	}
	
	/**
	 * Method get pipe output stream
	 * 
	 * @return instance of PipedOutputStream
	 */
	public PipedOutputStream getInput() {
		return currentdataForShell.input.getPos();
	}
	/**
	 * Method get  pipe input stream
	 * 
	 * @return instance of PipedInputStream
	 */
	public PipedInputStream getOutput() {
		return currentdataForShell.output.getPis();
	}
	
	/**
	 * Method create data for shell
	 * 
	 * @param key for map of DataForShell
	 */
	private void createNewInputOutput(String key) {
		DataForShell dataForShell = new DataForShell();
		dataForShell.input = new InputOutputPipe();
		dataForShell.output = new InputOutputPipe();
		dataForShell.text = createJTextArea();
		dataForShell.ir = new InputReader(dataForShell);
		
		currentdataForShell = dataForShell;
		dataForShellMap.put(key, dataForShell);

		dataForShell.ir.start();
	}

	/**
	 * Create main panel
	 * 
	 * @return instance of main panel
	 */
	private JPanel createMainPanel() {
		JPanel mainPanel = new JPanel(new BorderLayout());
			
		commandNewShell();
				
		mainPanel.add(mainJTabbedPane, BorderLayout.CENTER);

		return mainPanel;
	}
	
	/**
	 * Method for set shell ID
	 * 
	 * @param shellID shell ID
	 */
	public void setShellID(int shellID) {
		currentdataForShell.shellID = shellID;
	}
		
	/**
	 * Create tab for shell
	 */
	public void commandNewShell() {
		JPanel jPanel = new JPanel(new BorderLayout());
		jPanel.setName("id" + idShell);
	
		createNewInputOutput(jPanel.getName());
		
		jPanel.add(new JScrollPane(currentdataForShell.text), BorderLayout.CENTER);
		mainJTabbedPane.add("Shell " +  + idShell, jPanel);
		idShell++;
		
		mainJTabbedPane.setSelectedIndex(mainJTabbedPane.getTabCount()-1);
			
		currentdataForShell.idStartCommand = 0;
		currentdataForShell.panel = jPanel;
	}
	/**
	 * Close terminal
	 */
	public void shudown() {
		for (Map.Entry<String, DataForShell> entry : dataForShellMap.entrySet()) {		  
		    entry.getValue().ir.live = false;
		}
		dispose();
	}
	/**
	 * Close tab for shell
	 */
	public void closeShell() {		
		currentdataForShell.ir.live = false;
		try {
			currentdataForShell.input.getPos().close();
			currentdataForShell.input.getPis().close();
			currentdataForShell.output.getPos().close();
			currentdataForShell.output.getPis().close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		if (mainJTabbedPane.getTabCount() == 1) {
			setVisible(false);
			dispose();			
			return;
		}		
		dataForShellMap.remove(currentdataForShell);
		mainJTabbedPane.remove(mainJTabbedPane.getSelectedComponent());
		currentdataForShell = dataForShellMap.get(mainJTabbedPane.getSelectedComponent().getName()); 
		currentdataForShell.idStartCommand = currentdataForShell.text.getText().length();		currentdataForShell.text.setCaretPosition(currentdataForShell.idStartCommand);
		
	}
	/**
	 * Create text area for shell
	 * 
	 * @return instance of JTextArea
	 */
	private JTextArea createJTextArea() {		
		final JTextArea jTextArea = new JTextArea();	
		jTextArea.grabFocus();
		
		jTextArea.setBackground(Color.BLACK);
		jTextArea.setForeground(Color.WHITE);
		jTextArea.setCaretColor(Color.WHITE);
		jTextArea.setLineWrap(false);
		
		jTextArea.setHighlighter(null);
		jTextArea.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				if ((e.getKeyCode() == KeyEvent.VK_D) && (e.getModifiers() & KeyEvent.CTRL_MASK) != 0) {
					try {
						currentdataForShell.output.getPos().write(Character.ENCLOSING_MARK);
					} catch (IOException e1) {
						e1.printStackTrace();
					}
				}				
				if (e.getKeyChar() == KeyEvent.VK_ENTER) {
					try {
						jTextArea.append("\n");
						String command = jTextArea.getText().substring(currentdataForShell.idStartCommand);
						currentdataForShell.output.getPos().write(command.getBytes());						
						currentdataForShell.idStartCommand = jTextArea.getText().length();
						int shellID = currentdataForShell.shellID;
						Relation relation = RelationManager.getRelationManager().getRelation(shellID);
						
						if (relation != null) {
							relation.getHistory().remove(relation.getHistory().size()-1);
							relation.getHistory().add(command);
							currentdataForShell.idCommand = relation.getHistory().size();
							relation.getHistory().add("");
						}											
					} catch (IOException e1) {
						e1.printStackTrace();
					}
					e.consume();
					return;
				}
				if (e.getKeyChar() == KeyEvent.VK_BACK_SPACE) {
					if (jTextArea.getCaretPosition() == currentdataForShell.idStartCommand) {
						e.consume();
					}
					return;
				}
				if (e.getKeyCode() == KeyEvent.VK_UP) {	
					int shellID = currentdataForShell.shellID;
					Relation relation = RelationManager.getRelationManager().getRelation(shellID);	
					if (currentdataForShell.idCommand != 0) {
						currentdataForShell.text.setText(
								currentdataForShell.text.getText().substring(0, currentdataForShell.idStartCommand)
						);	
						if (relation != null) {
							currentdataForShell.text.append(relation.getHistory().get(--currentdataForShell.idCommand).replace('\n', ' ').trim());
						}
					}	
					return;
				}
				if (e.getKeyCode() == KeyEvent.VK_DOWN) {					
					int shellID = currentdataForShell.shellID;
					Relation relation = RelationManager.getRelationManager().getRelation(shellID);
					if (relation != null) {
						if (currentdataForShell.idCommand < relation.getHistory().size()-1) {
							currentdataForShell.text.setText(
									currentdataForShell.text.getText().substring(0, currentdataForShell.idStartCommand)
							);							
							currentdataForShell.text.append(relation.getHistory().get(++currentdataForShell.idCommand).replace('\n', ' ').trim());
						}
					}
					return;
				}
				String currentCommand = currentdataForShell.text.getText().substring(currentdataForShell.idStartCommand);
				int shellID = currentdataForShell.shellID;
				Relation relation = RelationManager.getRelationManager().getRelation(shellID);
				if (relation != null) {
					relation.getHistory().remove(relation.getHistory().size()-1);
					relation.getHistory().add(currentCommand);
				}
			}
		});
		
		jTextArea.addCaretListener(new CaretListener() {
			@Override
			public void caretUpdate(CaretEvent e) {
				if (e.getDot() == 0) {
					return;
				}			
				if (e.getDot() >= currentdataForShell.idStartCommand) {					
					currentdataForShell.posCaret = e.getDot();					
				} else  {
					jTextArea.setCaretPosition(currentdataForShell.posCaret);
					
				}
				jTextArea.getCaret().setVisible(true);	
			}
		});
		
		jTextArea.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				jTextArea.getCaret().setVisible(false);	
			}		
		});
		
		return jTextArea;
	}
	/**
	 * Class for reading input stream
	 *
	 */
	class InputReader extends Thread {
				
		DataForShell dataForShell;
		
		public InputReader(DataForShell dataForShell) {
			this.dataForShell = dataForShell;
		}
		
		boolean live = true;
		
		@Override
		public void run() {
			InputStreamReader is = new InputStreamReader(dataForShell.input.getPis());
			dataForShell.is = is;
			while(live) {
				try {
					int i;					
					if((i = is.read()) >= 0) {
						char c = (char)i;						
						dataForShell.text.append(c+"");
						dataForShell.idStartCommand = dataForShell.text.getText().length();
						dataForShell.text.setCaretPosition(currentdataForShell.idStartCommand);
					}					
				} catch (IOException e) {					
					//ignore - os is closing
				}
			}
		}
	}
}


